@@ -30,6 +30,7 @@
 #include <QRegExp>
 #include <QDebug>
 #include <QStandardPaths>
+#include <libfm-qt/filedialog.h>
 
 namespace PCManFM {
 
@@ -79,10 +80,20 @@ DesktopPreferencesDialog::DesktopPreferencesDialog(QWidget* parent, Qt::WindowFl
       i = 0;
   }
   ui.wallpaperMode->setCurrentIndex(i);
+  int mode = ui.wallpaperMode->itemData(ui.wallpaperMode->currentIndex()).toInt();
+
+  if (mode == DesktopWindow::WallpaperStretch || mode == DesktopWindow::WallpaperCenter
+      || mode == DesktopWindow::WallpaperFit || mode == DesktopWindow::WallpaperZoom) {
+    ui.perScreenWallpaper->setEnabled(true);
+  }
+  else
+    ui.perScreenWallpaper->setEnabled(false);
 
   connect(ui.browse, &QPushButton::clicked, this, &DesktopPreferencesDialog::onBrowseClicked);
   qDebug("wallpaper: %s", settings.wallpaper().toUtf8().data());
   ui.imageFile->setText(settings.wallpaper());
+  ui.transformImage->setChecked(settings.transformWallpaper());
+  ui.perScreenWallpaper->setChecked(settings.perScreenWallpaper());
 
   ui.slideShow->setChecked(settings.slideShowInterval() > 0);
   ui.imageFolder->setText(settings.wallpaperDir());
@@ -94,7 +105,7 @@ DesktopPreferencesDialog::DesktopPreferencesDialog(QWidget* parent, Qt::WindowFl
 
   for(std::size_t i = 0; i < G_N_ELEMENTS(iconSizes); ++i) {
     int size = iconSizes[i];
-    ui.iconSize->addItem(QString("%1 x %1").arg(size), size);
+    ui.iconSize->addItem(QStringLiteral("%1 x %1").arg(size), size);
     if(settings.desktopIconSize() == size)
       ui.iconSize->setCurrentIndex(i);
   }
@@ -111,14 +122,14 @@ DesktopPreferencesDialog::DesktopPreferencesDialog(QWidget* parent, Qt::WindowFl
   ui.computerBox->setChecked(ds.contains(QLatin1String("Computer")));
   ui.networkBox->setChecked(ds.contains(QLatin1String("Network")));
 
-  ui.showWmMenu->setChecked(settings.showWmMenu());
-
   connect(ui.buttonBox->button(QDialogButtonBox::Apply), &QPushButton::clicked,
           this, &DesktopPreferencesDialog::onApplyClicked);
 
   ui.hMargin->setValue(settings.desktopCellMargins().width());
   ui.vMargin->setValue(settings.desktopCellMargins().height());
   connect(ui.lockMargins, &QAbstractButton::clicked, this, &DesktopPreferencesDialog::lockMargins);
+
+  resize(sizeHint()); // show it compact
 }
 
 DesktopPreferencesDialog::~DesktopPreferencesDialog() {
@@ -158,6 +169,8 @@ void DesktopPreferencesDialog::applySettings()
       XdgDir::setDesktopDir(uiDesktopFolder.desktopFolder->text());
 
   settings.setWallpaper(ui.imageFile->text());
+  settings.setTransformWallpaper(ui.transformImage->isChecked());
+  settings.setPerScreenWallpaper(ui.perScreenWallpaper->isChecked());
   int mode = ui.wallpaperMode->itemData(ui.wallpaperMode->currentIndex()).toInt();
   settings.setWallpaperMode(mode);
 
@@ -190,8 +203,6 @@ void DesktopPreferencesDialog::applySettings()
   }
   settings.setDesktopShortcuts(ds);
 
-  settings.setShowWmMenu(ui.showWmMenu->isChecked());
-
   settings.setDesktopCellMargins(QSize(ui.hMargin->value(), ui.vMargin->value()));
 
   settings.save();
@@ -215,31 +226,64 @@ void DesktopPreferencesDialog::onWallpaperModeChanged(int index) {
   bool enable = (mode != DesktopWindow::WallpaperNone);
   ui.imageFile->setEnabled(enable);
   ui.browse->setEnabled(enable);
+  ui.transformImage->setEnabled(enable);
+
+  if (mode == DesktopWindow::WallpaperStretch || mode == DesktopWindow::WallpaperCenter
+      || mode == DesktopWindow::WallpaperFit || mode == DesktopWindow::WallpaperZoom) {
+    ui.perScreenWallpaper->setEnabled(true);
+  }
+  else
+    ui.perScreenWallpaper->setEnabled(false);
 }
 
 void DesktopPreferencesDialog::onBrowseClicked() {
-  QFileDialog dlg;
+  Settings& settings = static_cast<Application*>(qApp)->settings();
+
+  // use LXQt file dialog directly to set its view to thumbnail mode
+  Fm::FileDialog dlg(this);
+  dlg.resize(settings.wallpaperDialogSize());
+  dlg.setSplitterPos(settings.wallpaperDialogSplitterPos());
   dlg.setAcceptMode(QFileDialog::AcceptOpen);
   dlg.setFileMode(QFileDialog::ExistingFile);
-  // compose a name fileter from QImageReader
+  dlg.setWindowTitle(tr("Select Wallpaper"));
+  // compose a name filter from QImageReader
   QString filter;
   filter.reserve(256);
   filter = tr("Image Files");
-  filter += " (";
+  filter += QLatin1String(" (");
   const QList<QByteArray> formats = QImageReader::supportedImageFormats();
   for(const QByteArray& format : formats) {
-    filter += "*.";
-    filter += format.toLower();
-    filter += ' ';
+    filter += QLatin1String("*.");
+    filter += QString::fromUtf8(format.toLower());
+    filter += QLatin1Char(' ');
   }
-  filter += ')';
-  dlg.setNameFilter(filter);
-  dlg.setNameFilterDetailsVisible(false);
-  if(dlg.exec() == QDialog::Accepted) {
-    QString filename;
-    filename = dlg.selectedFiles().constFirst();
-    ui.imageFile->setText(filename);
+  filter += QLatin1Char(')');
+  dlg.setNameFilters(QStringList() << filter);
+  dlg.setViewMode(Fm::FolderView::ThumbnailMode);
+
+  // select an appropriate file
+  QString path = ui.imageFile->text();
+  if(path.isEmpty() || !QFile::exists(path)) {
+      dlg.setDirectory(QUrl::fromLocalFile(QStandardPaths::writableLocation(QStandardPaths::PicturesLocation)));
   }
+  else if(QFileInfo(path).isDir()) {
+      dlg.setDirectory(QUrl::fromLocalFile(path));
+  }
+  else { // the path is a file
+      dlg.setDirectory(QUrl::fromLocalFile(path.section(QLatin1String("/"), 0, -2)));
+      dlg.selectFile(QUrl::fromLocalFile(path));
+  }
+
+  connect(&dlg, &QDialog::finished, [this, &dlg, &settings](int result) {
+    if(result == QDialog::Accepted) {
+      QString filename;
+      filename = dlg.selectedFiles().constFirst().toLocalFile();
+      ui.imageFile->setText(filename);
+    }
+    settings.setWallpaperDialogSplitterPos(dlg.splitterPos());
+    settings.setWallpaperDialogSize(dlg.size());
+  });
+  dlg.exec();
 }
 
 void DesktopPreferencesDialog::onFolderBrowseClicked() {
@@ -247,7 +291,19 @@ void DesktopPreferencesDialog::onFolderBrowseClicked() {
   dlg.setAcceptMode(QFileDialog::AcceptOpen);
   dlg.setFileMode(QFileDialog::Directory);
   dlg.setOption(QFileDialog::ShowDirsOnly);
-  dlg.setDirectory(QDir::home().path());
+  dlg.setWindowTitle(tr("Select Wallpaper Folder"));
+
+  // select an appropriate dir
+  QString path = ui.imageFolder->text();
+  if(path.isEmpty() || !QFile::exists(path)) {
+      path = QStandardPaths::writableLocation(QStandardPaths::PicturesLocation);
+  }
+  else if(!QFileInfo(path).isDir()) {
+      path = path.section(QLatin1String("/"), 0, -2);
+  }
+  dlg.setDirectory(path.section(QLatin1String("/"), 0, -2));
+  dlg.selectFile(path);
+
   if(dlg.exec() == QDialog::Accepted) {
     QString foldername;
     foldername = dlg.selectedFiles().constFirst();
@@ -269,7 +325,7 @@ void DesktopPreferencesDialog::onBrowseDesktopFolderClicked()
 }
 
 void DesktopPreferencesDialog::selectPage(QString name) {
-  QWidget* page = findChild<QWidget*>(name + "Page");
+  QWidget* page = findChild<QWidget*>(name + QStringLiteral("Page"));
   if(page)
     ui.tabWidget->setCurrentWidget(page);
 }
