@@ -107,8 +107,8 @@ protected:
     virtual void paintEvent(QPaintEvent* event) override;
 
 protected Q_SLOTS:
-    void onOpenDirRequested(const Fm::FilePath& path, int target);
     void onDesktopPreferences();
+    void onCreatingShortcut();
     void selectAll();
     void toggleDesktop();
 
@@ -155,11 +155,15 @@ private:
     void createTrash();
     static void onTrashChanged(GFileMonitor* monitor, GFile* gf, GFile* other, GFileMonitorEvent evt, DesktopWindow* pThis);
     void trustOurDesktopShortcut(std::shared_ptr<const Fm::FileInfo> file);
-    bool isTrashCan(std::shared_ptr<const Fm::FileInfo> file);
+    bool isTrashCan(std::shared_ptr<const Fm::FileInfo> file) const;
+
+    QImage getWallpaperImage() const;
+
+    QModelIndex indexForPos(bool* isTrash, const QPoint& pos, const QRect& workArea, const QSize& grid) const;
 
 private:
     Fm::ProxyFolderModel* proxyModel_;
-    Fm::CachedFolderModel* model_;
+    Fm::FolderModel* model_;
     std::shared_ptr<Fm::Folder> folder_;
     Fm::FolderViewListView* listView_;
 
@@ -175,7 +179,6 @@ private:
     bool wallpaperRandomize_;
     QPixmap wallpaperPixmap_;
     Launcher fileLauncher_;
-    bool showWmMenu_;
     bool desktopHideItems_;
 
     int screenNum_;
@@ -188,6 +191,8 @@ private:
 
     QTimer* trashUpdateTimer_;
     GFileMonitor* trashMonitor_;
+
+    QStringList filesToTrust_;
 };
 
 }
