@@ -47,6 +47,7 @@
 #include <libfm-qt/core/fileinfo.h>
 #include <libfm-qt/mountoperation.h>
 #include "ui_about.h"
+#include "ui_shortcuts.h"
 #include "application.h"
 #include "bulkrename.h"
 
@@ -72,11 +73,9 @@ ViewFrame::ViewFrame(QWidget* parent):
     tabBar_->setElideMode(Qt::ElideRight);
     tabBar_->setExpanding(false);
     tabBar_->setMovable(true); // reorder the tabs by dragging
-#if QT_VERSION >= QT_VERSION_CHECK(5, 4, 0)
     // switch to the tab under the cursor during dnd.
     tabBar_->setChangeCurrentOnDrag(true);
     tabBar_->setAcceptDrops(true);
-#endif
     tabBar_->setContextMenuPolicy(Qt::CustomContextMenu);
 }
 
@@ -118,7 +117,7 @@ void ViewFrame::removeTopBar() {
 //======================================================================
 
 // static
-MainWindow* MainWindow::lastActive_ = nullptr;
+QPointer<MainWindow> MainWindow::lastActive_;
 
 MainWindow::MainWindow(Fm::FilePath path):
     QMainWindow(),
@@ -168,8 +167,15 @@ MainWindow::MainWindow(Fm::FilePath path):
     connect(ui.actionCloseOther, &QAction::triggered, this, &MainWindow::closeOtherTabs);
 
     ui.actionFilter->setChecked(settings.showFilter());
+    ui.actionShowThumbnails->setChecked(settings.showThumbnails());
+
+    // menu
+    ui.actionDelete->setText(settings.useTrash() ? tr("&Move to Trash") : tr("&Delete"));
+    ui.actionDelete->setIcon(settings.useTrash() ? QIcon::fromTheme(QStringLiteral("user-trash")) : QIcon::fromTheme(QStringLiteral("edit-delete")));
 
     // side pane
+    ui.sidePane->setVisible(settings.isSidePaneVisible());
+    ui.actionSidePane->setChecked(settings.isSidePaneVisible());
     ui.sidePane->setIconSize(QSize(settings.sidePaneIconSize(), settings.sidePaneIconSize()));
     ui.sidePane->setMode(settings.sidePaneMode());
     ui.sidePane->restoreHiddenPlaces(settings.getHiddenPlaces());
@@ -201,7 +207,7 @@ MainWindow::MainWindow(Fm::FilePath path):
 
     // use generic icons for view actions only if theme icons don't exist
     ui.actionIconView->setIcon(QIcon::fromTheme(QLatin1String("view-list-icons"), style()->standardIcon(QStyle::SP_FileDialogContentsView)));
-    ui.actionThumbnailView->setIcon(QIcon::fromTheme(QLatin1String("dialog-information"), style()->standardIcon(QStyle::SP_FileDialogInfoView)));
+    ui.actionThumbnailView->setIcon(QIcon::fromTheme(QLatin1String("view-preview"), style()->standardIcon(QStyle::SP_FileDialogInfoView)));
     ui.actionCompactView->setIcon(QIcon::fromTheme(QLatin1String("view-list-text"), style()->standardIcon(QStyle::SP_FileDialogListView)));
     ui.actionDetailedList->setIcon(QIcon::fromTheme(QLatin1String("view-list-details"), style()->standardIcon(QStyle::SP_FileDialogDetailedView)));
 
@@ -218,6 +224,7 @@ MainWindow::MainWindow(Fm::FilePath path):
     group->setExclusive(true);
     group->addAction(ui.actionByFileName);
     group->addAction(ui.actionByMTime);
+    group->addAction(ui.actionByDTime);
     group->addAction(ui.actionByFileSize);
     group->addAction(ui.actionByFileType);
     group->addAction(ui.actionByOwner);
@@ -248,7 +255,7 @@ MainWindow::MainWindow(Fm::FilePath path):
     menu->addMenu(ui.menu_Help);
     ui.actionMenu->setMenu(menu);
     if(ui.actionMenu->icon().isNull()) {
-        ui.actionMenu->setIcon(QIcon::fromTheme("applications-system"));
+        ui.actionMenu->setIcon(QIcon::fromTheme(QStringLiteral("applications-system")));
     }
     QToolButton* menuBtn = static_cast<QToolButton*>(ui.toolBar->widgetForAction(ui.actionMenu));
     menuBtn->setPopupMode(QToolButton::InstantPopup);
@@ -263,7 +270,19 @@ MainWindow::MainWindow(Fm::FilePath path):
     // create shortcuts
     QShortcut* shortcut;
     shortcut = new QShortcut(QKeySequence(Qt::Key_Escape), this);
-    connect(shortcut, &QShortcut::activated, this, &MainWindow::onResetFocus);
+    connect(shortcut, &QShortcut::activated, [this] {
+        if(currentPage()) {
+            currentPage()->clearFilter();
+            currentPage()->folderView()->childView()->setFocus();
+        }
+    });
+
+    shortcut = new QShortcut(QKeySequence(Qt::CTRL + Qt::Key_Escape), this);
+    connect(shortcut, &QShortcut::activated, [this] {
+        if(ui.sidePane->isVisible() && ui.sidePane->view()) {
+            ui.sidePane->view()->setFocus();
+        }
+    });
 
     shortcut = new QShortcut(QKeySequence(Qt::CTRL + Qt::Key_L), this);
     connect(shortcut, &QShortcut::activated, this, &MainWindow::focusPathEntry);
@@ -312,6 +331,10 @@ MainWindow::MainWindow(Fm::FilePath path):
     shortcut = new QShortcut(QKeySequence(Qt::CTRL + Qt::Key_F), this);
     connect(shortcut, &QShortcut::activated, ui.actionFindFiles, &QAction::trigger);
 
+    // in addition to Alt+Return, for convenience
+    shortcut = new QShortcut(Qt::ALT + Qt::Key_Enter, this);
+    connect(shortcut, &QShortcut::activated, this, &MainWindow::on_actionFileProperties_triggered);
+
     addViewFrame(path);
     if(splitView_) {
         // put the menu button on the right (there's no path bar/entry on the toolbar)
@@ -629,7 +652,6 @@ int MainWindow::addTabWithPage(TabPage* page, ViewFrame* viewFrame, Fm::FilePath
     int index = viewFrame->getStackedWidget()->addWidget(page);
     connect(page, &TabPage::titleChanged, this, &MainWindow::onTabPageTitleChanged);
     connect(page, &TabPage::statusChanged, this, &MainWindow::onTabPageStatusChanged);
-    connect(page, &TabPage::openDirRequested, this, &MainWindow::onTabPageOpenDirRequested);
     connect(page, &TabPage::sortFilterChanged, this, &MainWindow::onTabPageSortFilterChanged);
     connect(page, &TabPage::backwardRequested, this, &MainWindow::on_actionGoBack_triggered);
     connect(page, &TabPage::forwardRequested, this, &MainWindow::on_actionGoForward_triggered);
@@ -834,6 +856,10 @@ void MainWindow::on_actionShowHidden_triggered(bool checked) {
     ui.sidePane->setShowHidden(checked);
 }
 
+void MainWindow::on_actionShowThumbnails_triggered(bool checked) {
+    currentPage()->setShowThumbnails(checked);
+}
+
 void MainWindow::on_actionByFileName_triggered(bool /*checked*/) {
     currentPage()->sort(Fm::FolderModel::ColumnFileName, currentPage()->sortOrder());
 }
@@ -842,6 +868,10 @@ void MainWindow::on_actionByMTime_triggered(bool /*checked*/) {
     currentPage()->sort(Fm::FolderModel::ColumnFileMTime, currentPage()->sortOrder());
 }
 
+void MainWindow::on_actionByDTime_triggered(bool /*checked*/) {
+    currentPage()->sort(Fm::FolderModel::ColumnFileDTime, currentPage()->sortOrder());
+}
+
 void MainWindow::on_actionByOwner_triggered(bool /*checked*/) {
     currentPage()->sort(Fm::FolderModel::ColumnFileOwner, currentPage()->sortOrder());
 }
@@ -874,6 +904,10 @@ void MainWindow::on_actionFolderFirst_triggered(bool checked) {
     currentPage()->setSortFolderFirst(checked);
 }
 
+void MainWindow::on_actionHiddenLast_triggered(bool checked) {
+    currentPage()->setSortHiddenLast(checked);
+}
+
 void MainWindow::on_actionPreserveView_triggered(bool /*checked*/) {
     TabPage* page = currentPage();
     page->setCustomizedView(!page->hasCustomizedView());
@@ -966,8 +1000,19 @@ void MainWindow::on_actionAddToBookmarks_triggered() {
     if(page) {
         auto cwd = page->path();
         if(cwd) {
-            auto dispName = cwd.baseName();
-            bookmarks_->insert(cwd, dispName.get(), -1);
+            QString bookmarkName;
+            auto parent = cwd.parent();
+            if(!parent.isValid() || parent == cwd) { // a root path
+                bookmarkName = QString::fromUtf8(cwd.displayName().get());
+                auto parts = bookmarkName.split(QLatin1Char('/'), QString::SkipEmptyParts);
+                if(!parts.isEmpty()) {
+                    bookmarkName = parts.last();
+                }
+            }
+            else {
+                bookmarkName = QString::fromUtf8(cwd.baseName().get());
+            }
+            bookmarks_->insert(cwd, bookmarkName, -1);
         }
     }
 }
@@ -983,7 +1028,7 @@ void MainWindow::on_actionAbout_triggered() {
     public:
         explicit AboutDialog(QWidget* parent = 0, Qt::WindowFlags f = 0) : QDialog(parent, f) {
             ui.setupUi(this);
-            ui.version->setText(tr("Version: %1").arg(PCMANFM_QT_VERSION));
+            ui.version->setText(tr("Version: %1").arg(QStringLiteral(PCMANFM_QT_VERSION)));
         }
     private:
         Ui::AboutDialog ui;
@@ -992,6 +1037,24 @@ void MainWindow::on_actionAbout_triggered() {
     dialog.exec();
 }
 
+void MainWindow::on_actionHiddenShortcuts_triggered() {
+    class HiddenShortcutsDialog : public QDialog {
+    public:
+        explicit HiddenShortcutsDialog(QWidget* parent = 0, Qt::WindowFlags f = 0) : QDialog(parent, f) {
+            ui.setupUi(this);
+            ui.treeWidget->setRootIsDecorated(false);
+            ui.treeWidget->header()->setSectionResizeMode(QHeaderView::Stretch);
+            ui.treeWidget->header()->setSectionsClickable(true);
+            ui.treeWidget->sortByColumn(0, Qt::AscendingOrder);
+            ui.treeWidget->setSortingEnabled(true);
+        }
+    private:
+        Ui::HiddenShortcutsDialog ui;
+    };
+    HiddenShortcutsDialog dialog(this);
+    dialog.exec();
+}
+
 void MainWindow::on_actionIconView_triggered() {
     currentPage()->setViewMode(Fm::FolderView::IconMode);
 }
@@ -1187,14 +1250,36 @@ void MainWindow::updateViewMenuForCurrentPage() {
         modeAction->setChecked(true);
 
         // sort menu
+        // WARNING: Since libfm-qt may have a column that is not handled here,
+        // we should prevent a crash by setting all actions to null first and
+        // check their action group later.
         QAction* sortActions[Fm::FolderModel::NumOfColumns];
+        for(int i = 0; i < Fm::FolderModel::NumOfColumns; ++i) {
+            sortActions[i] = nullptr;
+        }
         sortActions[Fm::FolderModel::ColumnFileName] = ui.actionByFileName;
         sortActions[Fm::FolderModel::ColumnFileMTime] = ui.actionByMTime;
+        sortActions[Fm::FolderModel::ColumnFileDTime] = ui.actionByDTime;
         sortActions[Fm::FolderModel::ColumnFileSize] = ui.actionByFileSize;
         sortActions[Fm::FolderModel::ColumnFileType] = ui.actionByFileType;
         sortActions[Fm::FolderModel::ColumnFileOwner] = ui.actionByOwner;
         sortActions[Fm::FolderModel::ColumnFileGroup] = ui.actionByGroup;
-        sortActions[tabPage->sortColumn()]->setChecked(true);
+        if (auto group = ui.actionByFileName->actionGroup()) {
+            const auto actions = group->actions();
+            auto action = sortActions[tabPage->sortColumn()];
+            if(actions.contains(action)) {
+                action->setChecked(true);
+            }
+            else {
+                for(auto a : actions) {
+                    a->setChecked(false);
+                }
+            }
+        }
+
+        if(auto path = tabPage->path()) {
+            ui.actionByDTime->setVisible(strcmp(path.toString().get(), "trash:///") == 0);
+        }
 
         if(tabPage->sortOrder() == Qt::AscendingOrder) {
             ui.actionAscending->setChecked(true);
@@ -1204,6 +1289,7 @@ void MainWindow::updateViewMenuForCurrentPage() {
         }
         ui.actionCaseSensitive->setChecked(tabPage->sortCaseSensitive());
         ui.actionFolderFirst->setChecked(tabPage->sortFolderFirst());
+        ui.actionHiddenLast->setChecked(tabPage->sortHiddenLast());
     }
     updatingViewMenu_ = false;
 }
@@ -1287,6 +1373,10 @@ void MainWindow::updateUIForCurrentPage(bool setFocus) {
         }
     }
     ui.actionPaste->setEnabled(isWritable);
+    ui.menuCreateNew->setEnabled(isWritable);
+    // disable creation shortcuts too
+    ui.actionNewFolder->setEnabled(isWritable);
+    ui.actionNewBlankFile->setEnabled(isWritable);
 }
 
 void MainWindow::onStackedWidgetWidgetRemoved(int index) {
@@ -1358,6 +1448,9 @@ void MainWindow::onTabPageTitleChanged(QString title) {
                     }
                 }
                 ui.actionPaste->setEnabled(isWritable);
+                ui.menuCreateNew->setEnabled(isWritable);
+                ui.actionNewFolder->setEnabled(isWritable);
+                ui.actionNewBlankFile->setEnabled(isWritable);
             }
         }
     }
@@ -1392,25 +1485,6 @@ void MainWindow::onTabPageStatusChanged(int type, QString statusText) {
     updateEditSelectedActions();
 }
 
-void MainWindow::onTabPageOpenDirRequested(const Fm::FilePath& path, int target) {
-    TabPage* tabPage = static_cast<TabPage*>(sender());
-    if(ViewFrame* viewFrame = viewFrameForTabPage(tabPage)) {
-        switch(target) {
-        case OpenInCurrentTab:
-            chdir(path, viewFrame);
-            break;
-
-        case OpenInNewTab:
-            addTab(path, viewFrame);
-            break;
-
-        case OpenInNewWindow:
-            (new MainWindow(path))->show();
-            break;
-        }
-    }
-}
-
 void MainWindow::onTabPageSortFilterChanged() { // NOTE: This may be called from context menu too.
     TabPage* tabPage = static_cast<TabPage*>(sender());
     if(tabPage == currentPage()) {
@@ -1420,6 +1494,7 @@ void MainWindow::onTabPageSortFilterChanged() { // NOTE: This may be called from
             settings.setSortColumn(static_cast<Fm::FolderModel::ColumnId>(tabPage->sortColumn()));
             settings.setSortOrder(tabPage->sortOrder());
             settings.setSortFolderFirst(tabPage->sortFolderFirst());
+            settings.setSortHiddenLast(tabPage->sortHiddenLast());
             settings.setSortCaseSensitive(tabPage->sortCaseSensitive());
             settings.setShowHidden(tabPage->showHidden());
         }
@@ -1465,6 +1540,12 @@ void MainWindow::onSettingHiddenPlace(const QString& str, bool hide) {
     static_cast<Application*>(qApp)->settings().setHiddenPlace(str, hide);
 }
 
+void MainWindow::on_actionSidePane_triggered(bool checked) {
+    Application* app = static_cast<Application*>(qApp);
+    app->settings().showSidePane(checked);
+    ui.sidePane->setVisible(checked);
+}
+
 void MainWindow::onSplitterMoved(int pos, int /*index*/) {
     Application* app = static_cast<Application*>(qApp);
     app->settings().setSplitterPos(pos);
@@ -1598,8 +1679,8 @@ void MainWindow::on_actionPreferences_triggered() {
 
 // change some icons according to layout direction
 void MainWindow::setRTLIcons(bool isRTL) {
-    QIcon nxtIcn = QIcon::fromTheme("go-next");
-    QIcon prevIcn = QIcon::fromTheme("go-previous");
+    QIcon nxtIcn = QIcon::fromTheme(QStringLiteral("go-next"));
+    QIcon prevIcn = QIcon::fromTheme(QStringLiteral("go-previous"));
     if(isRTL) {
         ui.actionGoBack->setIcon(nxtIcn);
         ui.actionCloseLeft->setIcon(nxtIcn);
@@ -1646,7 +1727,7 @@ void MainWindow::onBackForwardContextMenu(QPoint pos) {
         const BrowseHistoryItem& item = history.at(i);
         auto path = item.path();
         auto name = path.displayName();
-        QAction* action = menu.addAction(name.get());
+        QAction* action = menu.addAction(QString::fromUtf8(name.get()));
         if(i == static_cast<size_t>(current)) {
             // make the current path bold and checked
             action->setCheckable(true);
@@ -1741,7 +1822,7 @@ void MainWindow::focusPathEntry() {
 }
 
 void MainWindow::dragEnterEvent(QDragEnterEvent* event) {
-    if(event->mimeData()->hasFormat("application/pcmanfm-qt-tab")
+    if(event->mimeData()->hasFormat(QStringLiteral("application/pcmanfm-qt-tab"))
        // ensure that the tab drag source is ours (and not a root window, for example)
        && lastActive_ && lastActive_->isActiveWindow()) {
         event->acceptProposedAction();
@@ -1749,7 +1830,7 @@ void MainWindow::dragEnterEvent(QDragEnterEvent* event) {
 }
 
 void MainWindow::dropEvent(QDropEvent* event) {
-    if(event->mimeData()->hasFormat("application/pcmanfm-qt-tab")) {
+    if(event->mimeData()->hasFormat(QStringLiteral("application/pcmanfm-qt-tab"))) {
         dropTab();
     }
     event->acceptProposedAction();
@@ -1811,7 +1892,7 @@ void MainWindow::updateFromSettings(Settings& settings) {
 
     // menu
     ui.actionDelete->setText(settings.useTrash() ? tr("&Move to Trash") : tr("&Delete"));
-    ui.actionDelete->setIcon(settings.useTrash() ? QIcon::fromTheme("user-trash") : QIcon::fromTheme("edit-delete"));
+    ui.actionDelete->setIcon(settings.useTrash() ? QIcon::fromTheme(QStringLiteral("user-trash")) : QIcon::fromTheme(QStringLiteral("edit-delete")));
 
     // side pane
     ui.sidePane->setIconSize(QSize(settings.sidePaneIconSize(), settings.sidePaneIconSize()));
@@ -1876,7 +1957,7 @@ void MainWindow::on_actionOpenAsRoot_triggered() {
         else {
             // show an error message and ask the user to set the command
             QMessageBox::critical(this, tr("Error"), tr("Switch user command is not set."));
-            app->preferences("advanced");
+            app->preferences(QStringLiteral("advanced"));
         }
     }
 }
@@ -1889,7 +1970,7 @@ void MainWindow::on_actionFindFiles_triggered() {
         for(auto& path: selectedPaths) {
             // FIXME: is it ok to use display name here?
             // This might be broken on filesystems with non-UTF-8 filenames.
-            paths.append(path.displayName().get());
+            paths.append(QString::fromUtf8(path.displayName().get()));
         }
     }
     else {
@@ -1911,7 +1992,7 @@ void MainWindow::on_actionCopyFullPath_triggered() {
     if(page) {
         auto paths = page->selectedFilePaths();
         if(paths.size() == 1) {
-            QApplication::clipboard()->setText(QString(paths.front().toString().get()), QClipboard::Clipboard);
+            QApplication::clipboard()->setText(QString::fromUtf8(paths.front().toString().get()), QClipboard::Clipboard);
         }
     }
 }
