@@ -21,6 +21,7 @@
 #define FM_MAIN_WINDOW_H
 
 #include "ui_main-win.h"
+#include <QPointer>
 #include <QMainWindow>
 #include <QListView>
 #include <QSortFilterProxyModel>
@@ -140,11 +141,13 @@ protected Q_SLOTS:
 
     void on_actionGo_triggered();
     void on_actionShowHidden_triggered(bool check);
+    void on_actionShowThumbnails_triggered(bool check);
     void on_actionSplitView_triggered(bool check);
     void on_actionPreserveView_triggered(bool checked);
 
     void on_actionByFileName_triggered(bool checked);
     void on_actionByMTime_triggered(bool checked);
+    void on_actionByDTime_triggered(bool checked);
     void on_actionByOwner_triggered(bool checked);
     void on_actionByGroup_triggered(bool checked);
     void on_actionByFileType_triggered(bool checked);
@@ -152,6 +155,7 @@ protected Q_SLOTS:
     void on_actionAscending_triggered(bool checked);
     void on_actionDescending_triggered(bool checked);
     void on_actionFolderFirst_triggered(bool checked);
+    void on_actionHiddenLast_triggered(bool checked);
     void on_actionCaseSensitive_triggered(bool checked);
     void on_actionFilter_triggered(bool checked);
     void on_actionUnfilter_triggered();
@@ -174,6 +178,7 @@ protected Q_SLOTS:
     void on_actionFindFiles_triggered();
 
     void on_actionAbout_triggered();
+    void on_actionHiddenShortcuts_triggered();
 
     void onBookmarkActionTriggered();
 
@@ -189,7 +194,6 @@ protected Q_SLOTS:
 
     void onTabPageTitleChanged(QString title);
     void onTabPageStatusChanged(int type, QString statusText);
-    void onTabPageOpenDirRequested(const Fm::FilePath &path, int target);
     void onTabPageSortFilterChanged();
 
     void onSidePaneChdirRequested(int type, const Fm::FilePath &path);
@@ -198,6 +202,7 @@ protected Q_SLOTS:
     void onSidePaneOpenFolderInTerminalRequested(const Fm::FilePath &path);
     void onSidePaneCreateNewFolderRequested(const Fm::FilePath &path);
     void onSidePaneModeChanged(Fm::SidePane::Mode mode);
+    void on_actionSidePane_triggered(bool check);
     void onSplitterMoved(int pos, int index);
     void onResetFocus();
 
@@ -264,7 +269,7 @@ private:
     // not from another window. So, we get the mode at the start and keep it.
     bool splitView_;
 
-    static MainWindow* lastActive_;
+    static QPointer<MainWindow> lastActive_;
 };
 
 }
