@@ -58,7 +58,7 @@ static void findIconThemesInDir(QHash<QString, QString>& iconThemes, QString dir
     const QStringList subDirs = dir.entryList(QDir::AllDirs);
     GKeyFile* kf = g_key_file_new();
     for(const QString& subDir : subDirs) {
-        QString indexFile = dirName % '/' % subDir % "/index.theme";
+        QString indexFile = dirName + QLatin1Char('/') + subDir + QStringLiteral("/index.theme");
         if(g_key_file_load_from_file(kf, indexFile.toLocal8Bit().constData(), GKeyFileFlags(0), nullptr)) {
             // FIXME: skip hidden ones
             // icon theme must have this key, so it has icons if it has this key
@@ -66,7 +66,7 @@ static void findIconThemesInDir(QHash<QString, QString>& iconThemes, QString dir
             if(g_key_file_has_key(kf, "Icon Theme", "Directories", nullptr)) {
                 char* dispName = g_key_file_get_locale_string(kf, "Icon Theme", "Name", nullptr, nullptr);
                 // char* comment = g_key_file_get_locale_string(kf, "Icon Theme", "Comment", nullptr, nullptr);
-                iconThemes[subDir] = dispName;
+                iconThemes[subDir] = QString::fromUtf8(dispName);
                 g_free(dispName);
             }
         }
@@ -80,15 +80,15 @@ void PreferencesDialog::initIconThemes(Settings& settings) {
         // load xdg icon themes and select the current one
         QHash<QString, QString> iconThemes;
         // user customed icon themes
-        findIconThemesInDir(iconThemes, QString(g_get_home_dir()) % "/.icons");
+        findIconThemesInDir(iconThemes, QString::fromUtf8(g_get_home_dir()) + QStringLiteral("/.icons"));
 
         // search for icons in system data dir
         const char* const* dataDirs = g_get_system_data_dirs();
         for(const char* const* dataDir = dataDirs; *dataDir; ++dataDir) {
-            findIconThemesInDir(iconThemes, QString(*dataDir) % "/icons");
+            findIconThemesInDir(iconThemes, QString::fromUtf8(*dataDir) + QStringLiteral("/icons"));
         }
 
-        iconThemes.remove("hicolor"); // remove hicolor, which is only a fallback
+        iconThemes.remove(QStringLiteral("hicolor")); // remove hicolor, which is only a fallback
         QHash<QString, QString>::const_iterator it;
         for(it = iconThemes.constBegin(); it != iconThemes.constEnd(); ++it) {
             ui.iconTheme->addItem(it.value(), it.key());
@@ -123,8 +123,8 @@ void PreferencesDialog::initArchivers(Settings& settings) {
     auto& allArchivers = Fm::Archiver::allArchivers();
     for(int i = 0; i < int(allArchivers.size()); ++i) {
         auto& archiver = allArchivers[i];
-        ui.archiver->addItem(archiver->program(), QString(archiver->program()));
-        if(archiver->program() == settings.archiver()) {
+        ui.archiver->addItem(QString::fromUtf8(archiver->program()), QString::fromUtf8(archiver->program()));
+        if(QString::fromUtf8(archiver->program()) == settings.archiver()) {
             ui.archiver->setCurrentIndex(i);
         }
     }
@@ -135,7 +135,7 @@ void PreferencesDialog::initDisplayPage(Settings& settings) {
     // icon sizes
     int i = 0;
     for (const auto & size : Settings::iconSizes(Settings::Big)) {
-        ui.bigIconSize->addItem(QString("%1 x %1").arg(size), size);
+        ui.bigIconSize->addItem(QStringLiteral("%1 x %1").arg(size), size);
         if(settings.bigIconSize() == size) {
             ui.bigIconSize->setCurrentIndex(i);
         }
@@ -143,7 +143,7 @@ void PreferencesDialog::initDisplayPage(Settings& settings) {
     }
     i = 0;
     for (const auto & size : Settings::iconSizes(Settings::Small)) {
-        QString text = QString("%1 x %1").arg(size);
+        QString text = QStringLiteral("%1 x %1").arg(size);
         ui.smallIconSize->addItem(text, size);
         if(settings.smallIconSize() == size) {
             ui.smallIconSize->setCurrentIndex(i);
@@ -157,7 +157,7 @@ void PreferencesDialog::initDisplayPage(Settings& settings) {
     }
     i = 0;
     for (const auto & size : Settings::iconSizes(Settings::Thumbnail)) {
-        ui.thumbnailIconSize->addItem(QString("%1 x %1").arg(size), size);
+        ui.thumbnailIconSize->addItem(QStringLiteral("%1 x %1").arg(size), size);
         if(settings.thumbnailIconSize() == size) {
             ui.thumbnailIconSize->setCurrentIndex(i);
         }
@@ -190,6 +190,7 @@ void PreferencesDialog::initUiPage(Settings& settings) {
 void PreferencesDialog::initBehaviorPage(Settings& settings) {
     ui.singleClick->setChecked(settings.singleClick());
     ui.autoSelectionDelay->setValue(double(settings.autoSelectionDelay()) / 1000);
+    ui.ctrlRightClick->setChecked(settings.ctrlRightClick());
 
     ui.bookmarkOpenMethod->setCurrentIndex(settings.bookmarkOpenMethod());
 
@@ -223,6 +224,7 @@ void PreferencesDialog::initBehaviorPage(Settings& settings) {
     ui.confirmTrash->setChecked(settings.confirmTrash());
     ui.quickExec->setChecked(settings.quickExec());
     ui.selectNewFiles->setChecked(settings.selectNewFiles());
+    ui.singleWindowMode->setChecked(settings.singleWindowMode());
 
     // app restart warning
     connect(ui.quickExec, &QAbstractButton::toggled, [this, &settings] (bool checked) {
@@ -251,7 +253,7 @@ void PreferencesDialog::initVolumePage(Settings& settings) {
 void PreferencesDialog::initTerminals(Settings& settings) {
     // load the known terminal list from the terminal.list file of libfm
     for(auto& terminal: Fm::allKnownTerminals()) {
-        ui.terminal->addItem(terminal.get());
+        ui.terminal->addItem(QString::fromUtf8(terminal.get()));
     }
     ui.terminal->setEditText(settings.terminal());
 }
@@ -320,6 +322,7 @@ void PreferencesDialog::applyUiPage(Settings& settings) {
 void PreferencesDialog::applyBehaviorPage(Settings& settings) {
     settings.setSingleClick(ui.singleClick->isChecked());
     settings.setAutoSelectionDelay(int(ui.autoSelectionDelay->value() * 1000));
+    settings.setCtrlRightClick(ui.ctrlRightClick->isChecked());
 
     settings.setBookmarkOpenMethod(OpenDirTargetType(ui.bookmarkOpenMethod->currentIndex()));
 
@@ -336,6 +339,7 @@ void PreferencesDialog::applyBehaviorPage(Settings& settings) {
     settings.setConfirmTrash(ui.confirmTrash->isChecked());
     settings.setQuickExec(ui.quickExec->isChecked());
     settings.setSelectNewFiles(ui.selectNewFiles->isChecked());
+    settings.setSingleWindowMode(ui.singleWindowMode->isChecked());
 }
 
 void PreferencesDialog::applyThumbnailPage(Settings& settings) {
@@ -384,7 +388,7 @@ void PreferencesDialog::accept() {
 
 void PreferencesDialog::selectPage(QString name) {
     if(!name.isEmpty()) {
-        QWidget* page = findChild<QWidget*>(name + "Page");
+        QWidget* page = findChild<QWidget*>(name + QStringLiteral("Page"));
         if(page) {
             int index = ui.stackedWidget->indexOf(page);
             ui.listWidget->setCurrentRow(index);
