@@ -43,11 +43,14 @@ class FolderSettings {
 public:
     FolderSettings():
         isCustomized_(false),
+        // NOTE: The default values of the following variables should be
+        // the same as those of their corresponding variables in Settings:
         sortOrder_(Qt::AscendingOrder),
         sortColumn_(Fm::FolderModel::ColumnFileName),
         viewMode_(Fm::FolderView::IconMode),
         showHidden_(false),
         sortFolderFirst_(true),
+        sortHiddenLast_(false),
         sortCaseSensitive_(true) {
     }
 
@@ -91,6 +94,14 @@ public:
         sortFolderFirst_ = value;
     }
 
+    bool sortHiddenLast() const {
+        return sortHiddenLast_;
+    }
+
+    void setSortHiddenLast(bool value) {
+        sortHiddenLast_ = value;
+    }
+
     bool showHidden() const {
         return showHidden_;
     }
@@ -114,6 +125,7 @@ private:
     Fm::FolderView::ViewMode viewMode_;
     bool showHidden_;
     bool sortFolderFirst_;
+    bool sortHiddenLast_;
     bool sortCaseSensitive_;
     // columns?
 };
@@ -131,7 +143,7 @@ public:
     Settings();
     virtual ~Settings();
 
-    bool load(QString profile = "default");
+    bool load(QString profile = QStringLiteral("default"));
     bool save(QString profile = QString());
 
     bool loadFile(QString filePath);
@@ -163,6 +175,14 @@ public:
         fallbackIconThemeName_ = iconThemeName;
     }
 
+    bool singleWindowMode() const {
+        return singleWindowMode_;
+    }
+
+    void setSingleWindowMode(bool singleWindowMode) {
+        singleWindowMode_ = singleWindowMode;
+    }
+
     OpenDirTargetType bookmarkOpenMethod() {
         return bookmarkOpenMethod_;
     }
@@ -241,6 +261,22 @@ public:
         wallpaper_ = wallpaper;
     }
 
+    QSize wallpaperDialogSize() const {
+        return wallpaperDialogSize_;
+    }
+
+    void setWallpaperDialogSize(const QSize& size) {
+        wallpaperDialogSize_ = size;
+    }
+
+    int wallpaperDialogSplitterPos() const {
+        return wallpaperDialogSplitterPos_;
+    }
+
+    void setWallpaperDialogSplitterPos(int pos) {
+        wallpaperDialogSplitterPos_ = pos;
+    }
+
     QString wallpaperDir() const {
         return wallpaperDir_;
     }
@@ -273,6 +309,22 @@ public:
         wallpaperRandomize_ = randomize;
     }
 
+    bool transformWallpaper() const {
+       return transformWallpaper_;
+    }
+
+    void setTransformWallpaper(bool tr) {
+        transformWallpaper_ = tr;
+    }
+
+    bool perScreenWallpaper() const {
+       return perScreenWallpaper_;
+    }
+
+    void setPerScreenWallpaper(bool tr) {
+        perScreenWallpaper_ = tr;
+    }
+
     const QColor& desktopBgColor() const {
         return desktopBgColor_;
     }
@@ -321,14 +373,6 @@ public:
         desktopShortcuts_ = list;
     }
 
-    bool showWmMenu() const {
-        return showWmMenu_;
-    }
-
-    void setShowWmMenu(bool value) {
-        showWmMenu_ = value;
-    }
-
     bool desktopShowHidden() const {
         return desktopShowHidden_;
     }
@@ -369,6 +413,14 @@ public:
         desktopSortFolderFirst_ = desktopFolderFirst;
     }
 
+    bool desktopSortHiddenLast() const {
+        return desktopSortHiddenLast_;
+    }
+
+    void setDesktopSortHiddenLast(bool desktopHiddenLast) {
+        desktopSortHiddenLast_ = desktopHiddenLast;
+    }
+
     bool alwaysShowTabs() const {
         return alwaysShowTabs_;
     }
@@ -456,6 +508,14 @@ public:
         splitterPos_ = splitterPos;
     }
 
+    bool isSidePaneVisible() const {
+        return sidePaneVisible_;
+    }
+
+    void showSidePane(bool show) {
+        sidePaneVisible_ = show;
+    }
+
     Fm::SidePane::Mode sidePaneMode() const {
         return sidePaneMode_;
     }
@@ -599,6 +659,14 @@ public:
         sortFolderFirst_ = folderFirst;
     }
 
+    bool sortHiddenLast() const {
+        return sortHiddenLast_;
+    }
+
+    void setSortHiddenLast(bool hiddenLast) {
+        sortHiddenLast_ = hiddenLast;
+    }
+
     bool showFilter() const {
         return showFilter_;
     }
@@ -632,6 +700,14 @@ public:
         autoSelectionDelay_ = value;
     }
 
+    bool ctrlRightClick() const {
+        return ctrlRightClick_;
+    }
+
+    void setCtrlRightClick(bool value) {
+        ctrlRightClick_ = value;
+    }
+
     bool useTrash() const {
         if(!supportTrash_) {
             return false;
@@ -918,6 +994,7 @@ private:
     QString fallbackIconThemeName_;
     bool useFallbackIconTheme_;
 
+    bool singleWindowMode_;
     OpenDirTargetType bookmarkOpenMethod_;
     QString suCommand_;
     QString terminal_;
@@ -928,23 +1005,27 @@ private:
 
     int wallpaperMode_;
     QString wallpaper_;
+    QSize wallpaperDialogSize_;
+    int wallpaperDialogSplitterPos_;
     QString lastSlide_;
     QString wallpaperDir_;
     int slideShowInterval_;
     bool wallpaperRandomize_;
+    bool transformWallpaper_;
+    bool perScreenWallpaper_;
     QColor desktopBgColor_;
     QColor desktopFgColor_;
     QColor desktopShadowColor_;
     QFont desktopFont_;
     int desktopIconSize_;
     QStringList desktopShortcuts_;
-    bool showWmMenu_;
 
     bool desktopShowHidden_;
     bool desktopHideItems_;
     Qt::SortOrder desktopSortOrder_;
     Fm::FolderModel::ColumnId desktopSortColumn_;
     bool desktopSortFolderFirst_;
+    bool desktopSortHiddenLast_;
 
     bool alwaysShowTabs_;
     bool showTabClose_;
@@ -955,6 +1036,7 @@ private:
     int lastWindowHeight_;
     bool lastWindowMaximized_;
     int splitterPos_;
+    bool sidePaneVisible_;
     Fm::SidePane::Mode sidePaneMode_;
     bool showMenuBar_;
     bool splitView_;
@@ -964,6 +1046,7 @@ private:
     Qt::SortOrder sortOrder_;
     Fm::FolderModel::ColumnId sortColumn_;
     bool sortFolderFirst_;
+    bool sortHiddenLast_;
     bool sortCaseSensitive_;
     bool showFilter_;
     bool pathBarButtons_;
@@ -971,6 +1054,7 @@ private:
     // settings for use with libfm
     bool singleClick_;
     int autoSelectionDelay_;
+    bool ctrlRightClick_;
     bool useTrash_;
     bool confirmDelete_;
     bool noUsbTrash_; // do not trash files on usb removable devices
