@@ -59,7 +59,7 @@ bool ProxyFilter::filterAcceptsRow(const Fm::ProxyFolderModel* model, const std:
 FilterEdit::FilterEdit(QWidget* parent) : QLineEdit(parent) {
     setClearButtonEnabled(true);
     if(QToolButton *clearButton = findChild<QToolButton*>()) {
-        clearButton->setToolTip(tr("Clear text (Ctrl+K)"));
+        clearButton->setToolTip(tr("Clear text (Ctrl+K or Esc)"));
     }
 }
 
@@ -128,7 +128,6 @@ TabPage::TabPage(QWidget* parent):
     folderView_->setMargins(settings.folderViewCellMargins());
     folderView_->setShadowHidden(settings.shadowHidden());
     // newView->setColumnWidth(Fm::FolderModel::ColumnName, 200);
-    connect(folderView_, &View::openDirRequested, this, &TabPage::openDirRequested);
     connect(folderView_, &View::selChanged, this, &TabPage::onSelChanged);
     connect(folderView_, &View::clickedBack, this, &TabPage::backwardRequested);
     connect(folderView_, &View::clickedForward, this, &TabPage::forwardRequested);
@@ -230,10 +229,25 @@ void TabPage::backspacePressed() {
 
 void TabPage::onFilterStringChanged(QString str) {
     if(filterBar_ && str != getFilterStr()) {
+        bool transientFilterBar = !static_cast<Application*>(qApp)->settings().showFilter();
+
+        // with a transient filter-bar, let the current index be selected by Qt
+        // if the first pressed key is a space
+        if(transientFilterBar && !filterBar_->isVisibleTo(this)
+           && folderView()->childView()->hasFocus()
+           && str == QString(QChar(QChar::Space))) {
+            QModelIndex index = folderView_->selectionModel()->currentIndex();
+            if (index.isValid() && !folderView_->selectionModel()->isSelected(index)) {
+                filterBar_->clear();
+                folderView_->childView()->scrollTo(index, QAbstractItemView::EnsureVisible);
+                return;
+            }
+        }
+
         setFilterStr(str);
         applyFilter();
         // show/hide the transient filter-bar appropriately
-        if(!static_cast<Application*>(qApp)->settings().showFilter()) {
+        if(transientFilterBar) {
             if(filterBar_->isVisibleTo(this)) { // the page itself may be in an inactive tab
                 if(str.isEmpty()) {
                     // focus the view BEFORE hiding the filter-bar to avoid redundant "FocusIn" events;
@@ -305,6 +319,13 @@ void TabPage::onUiUpdated() {
             folderView_->childView()->setCurrentIndex(index);
         }
     }
+    else { // set the first item as current
+        QModelIndex firstIndx = proxyModel_->index(0, 0);
+        if (firstIndx.isValid()) {
+            folderView_->selectionModel()->setCurrentIndex(firstIndx, QItemSelectionModel::NoUpdate);
+        }
+    }
+
     if(folderModel_) {
         // update selection statusbar info when needed
         connect(folderModel_, &Fm::FolderModel::fileSizeChanged, this, &TabPage::onFileSizeChanged);
@@ -336,6 +357,13 @@ void TabPage::onFilesAdded(Fm::FileInfoList files) {
             selectionTimer_->start(200);
         }
     }
+    else if (!folderView_->selectionModel()->currentIndex().isValid()) {
+        // set the first item as current if there is no current item
+        QModelIndex firstIndx = proxyModel_->index(0, 0);
+        if (firstIndx.isValid()) {
+            folderView_->selectionModel()->setCurrentIndex(firstIndx, QItemSelectionModel::NoUpdate);
+        }
+    }
 }
 
 void TabPage::onFolderFinishLoading() {
@@ -427,8 +455,8 @@ void TabPage::onFolderFsInfo() {
     QString& msg = statusText_[StatusTextFSInfo];
     if(folder_->getFilesystemInfo(&total, &free)) {
         msg = tr("Free space: %1 (Total: %2)")
-              .arg(formatFileSize(free, fm_config->si_unit))
-              .arg(formatFileSize(total, fm_config->si_unit));
+              .arg(formatFileSize(free, fm_config->si_unit),
+                   formatFileSize(total, fm_config->si_unit));
     }
     else {
         msg.clear();
@@ -449,7 +477,7 @@ QString TabPage::formatStatusText() {
         }
         auto fi = folder_->info();
         if (fi && fi->isSymlink()) {
-            text += QString(" %2(%1)")
+            text += QStringLiteral(" %2(%1)")
                     .arg(encloseWithBidiMarks(tr("Link to") + QChar(QChar::Space) + QString::fromStdString(fi->target())),
                     (layoutDirection() == Qt::RightToLeft) ? QChar(0x200f) : QChar(0x200e));
         }
@@ -534,7 +562,7 @@ void TabPage::chdir(Fm::FilePath newPath, bool addHistory) {
         freeFolder();
     }
 
-    Q_EMIT titleChanged(newPath.baseName().get());  // FIXME: display name
+    Q_EMIT titleChanged(QString::fromUtf8(newPath.baseName().get()));  // FIXME: display name
 
     folder_ = Fm::Folder::fromPath(newPath);
     if(addHistory) {
@@ -554,7 +582,16 @@ void TabPage::chdir(Fm::FilePath newPath, bool addHistory) {
 
     Settings& settings = static_cast<Application*>(qApp)->settings();
     folderModel_ = CachedFolderModel::modelFromFolder(folder_);
-    folderModel_->setShowFullName(settings.showFullNames());
+    // always show display names in special places because real names may be unusual
+    // (e.g., real names of trashed files may contain trash path with backslash)
+    if(!settings.showFullNames()
+       || newPath.hasUriScheme("menu") || newPath.hasUriScheme("trash")
+       || newPath.hasUriScheme("network") || newPath.hasUriScheme("computer")) {
+        folderModel_->setShowFullName(false);
+    }
+    else {
+        folderModel_->setShowFullName(true);
+    }
 
     // folderSettings_ will be set by saveFolderSorting() when the sort filter is changed below
     // (and also by setViewMode()); here, we only need to know whether it should be saved
@@ -564,6 +601,7 @@ void TabPage::chdir(Fm::FilePath newPath, bool addHistory) {
     // set sorting
     proxyModel_->sort(folderSettings.sortColumn(), folderSettings.sortOrder());
     proxyModel_->setFolderFirst(folderSettings.sortFolderFirst());
+    proxyModel_->setHiddenLast(folderSettings.sortHiddenLast());
     proxyModel_->setShowHidden(folderSettings.showHidden());
     proxyModel_->setSortCaseSensitivity(folderSettings.sortCaseSensitive() ? Qt::CaseSensitive : Qt::CaseInsensitive);
     proxyModel_->setSourceModel(folderModel_);
@@ -622,34 +660,38 @@ void TabPage::onSelChanged() {
         if(numSel == 1) { /* only one file is selected (also, tell if it is a symlink)*/
             auto& fi = files.front();
             if(!fi->isDir()) {
+                QString name = static_cast<Application*>(qApp)->settings().showFullNames()
+                               && strcmp(fi->dirPath().uriScheme().get(), "menu") != 0
+                                   ? QString::fromStdString(fi->name())
+                                   : fi->displayName();
                 if(fi->isSymlink()) {
-                    msg = QString("%5\"%1\" %5(%2) %5%3 %5(%4)")
-                          .arg(encloseWithBidiMarks(fi->displayName()),
+                    msg = QStringLiteral("%5\"%1\" %5(%2) %5%3 %5(%4)")
+                          .arg(encloseWithBidiMarks(name),
                           encloseWithBidiMarks(Fm::formatFileSize(fi->size(), fm_config->si_unit)),
-                          encloseWithBidiMarks(fi->mimeType()->desc()),
+                          encloseWithBidiMarks(QString::fromUtf8(fi->mimeType()->desc())),
                           encloseWithBidiMarks(tr("Link to") + QChar(QChar::Space) + QString::fromStdString(fi->target())),
                           (layoutDirection() == Qt::RightToLeft) ? QChar(0x200f) : QChar(0x200e));
                 }
                 else {
-                    msg = QString("%4\"%1\" %4(%2) %4%3")
-                          .arg(encloseWithBidiMarks(fi->displayName()),
+                    msg = QStringLiteral("%4\"%1\" %4(%2) %4%3")
+                          .arg(encloseWithBidiMarks(name),
                           encloseWithBidiMarks(Fm::formatFileSize(fi->size(), fm_config->si_unit)), // FIXME: deprecate fm_config
-                          encloseWithBidiMarks(fi->mimeType()->desc()),
+                          encloseWithBidiMarks(QString::fromUtf8(fi->mimeType()->desc())),
                           (layoutDirection() == Qt::RightToLeft) ? QChar(0x200f) : QChar(0x200e));
                 }
             }
             else {
                 if(fi->isSymlink()) {
-                    msg = QString("%4\"%1\" %4%2 %4(%3)")
+                    msg = QStringLiteral("%4\"%1\" %4%2 %4(%3)")
                           .arg(encloseWithBidiMarks(fi->displayName()),
-                          encloseWithBidiMarks(fi->mimeType()->desc()),
+                          encloseWithBidiMarks(QString::fromUtf8(fi->mimeType()->desc())),
                           encloseWithBidiMarks(tr("Link to") + QChar(QChar::Space) + QString::fromStdString(fi->target())),
                           (layoutDirection() == Qt::RightToLeft) ? QChar(0x200f) : QChar(0x200e));
                 }
                 else {
-                    msg = QString("%3\"%1\" %3%2")
+                    msg = QStringLiteral("%3\"%1\" %3%2")
                           .arg(encloseWithBidiMarks(fi->displayName()),
-                          encloseWithBidiMarks(fi->mimeType()->desc()),
+                          encloseWithBidiMarks(QString::fromUtf8(fi->mimeType()->desc())),
                           (layoutDirection() == Qt::RightToLeft) ? QChar(0x200f) : QChar(0x200e));
                 }
             }
@@ -671,7 +713,7 @@ void TabPage::onSelChanged() {
                     sum += fi->size();
                 }
                 if(sum >= 0) {
-                    msg += QString(" (%1)").arg(Fm::formatFileSize(sum, fm_config->si_unit)); // FIXME: deprecate fm_config
+                    msg += QStringLiteral(" (%1)").arg(Fm::formatFileSize(sum, fm_config->si_unit)); // FIXME: deprecate fm_config
                 }
                 /* FIXME: should we support statusbar plugins as in the gtk+ version? */
             }
@@ -769,6 +811,12 @@ void TabPage::setSortFolderFirst(bool value) {
     }
 }
 
+void TabPage::setSortHiddenLast(bool value) {
+    if(proxyModel_) {
+        proxyModel_->setHiddenLast(value);
+    }
+}
+
 void TabPage::setSortCaseSensitive(bool value) {
     if(proxyModel_) {
         proxyModel_->setSortCaseSensitivity(value ? Qt::CaseSensitive : Qt::CaseInsensitive);
@@ -781,6 +829,14 @@ void TabPage::setShowHidden(bool showHidden) {
     }
 }
 
+void TabPage::setShowThumbnails(bool showThumbnails) {
+    Settings& settings = static_cast<Application*>(qApp)->settings();
+    settings.setShowThumbnails(showThumbnails);
+    if(proxyModel_) {
+        proxyModel_->setShowThumbnails(showThumbnails);
+    }
+}
+
 void TabPage::saveFolderSorting() {
     if (proxyModel_ == nullptr) {
         return;
@@ -788,6 +844,7 @@ void TabPage::saveFolderSorting() {
     folderSettings_.setSortOrder(proxyModel_->sortOrder());
     folderSettings_.setSortColumn(static_cast<Fm::FolderModel::ColumnId>(proxyModel_->sortColumn()));
     folderSettings_.setSortFolderFirst(proxyModel_->folderFirst());
+    folderSettings_.setSortHiddenLast(proxyModel_->hiddenLast());
     folderSettings_.setSortCaseSensitive(proxyModel_->sortCaseSensitivity());
     if(folderSettings_.showHidden() != proxyModel_->showHidden()) {
         folderSettings_.setShowHidden(proxyModel_->showHidden());
@@ -803,12 +860,32 @@ void TabPage::applyFilter() {
     if(proxyModel_ == nullptr) {
         return;
     }
+
     int prevSelSize = folderView_->selectionModel()->selectedIndexes().size();
+    bool selectFirst = false;
+
     proxyModel_->updateFilters();
+
+    QModelIndex firstIndx = proxyModel_->index(0, 0);
+    if (firstIndx.isValid()) {
+        if(proxyFilter_->getFilterStr().isEmpty()) {
+            // if the filter is removed and there is no current item, set the first item as current
+            if (!folderView_->selectionModel()->currentIndex().isValid()) {
+                folderView_->selectionModel()->setCurrentIndex(firstIndx, QItemSelectionModel::NoUpdate);
+            }
+        }
+        else if (!folderView_->hasSelection()) {
+            // if there is a filter but no selection, select the first item
+            folderView_->childView()->setCurrentIndex(firstIndx);
+            selectFirst = true;
+        }
+    }
+
     // if some selected files are filtered out, "View::selChanged()" won't be emitted
-    if(prevSelSize > folderView_->selectionModel()->selectedIndexes().size()) {
+    if(!selectFirst && prevSelSize > folderView_->selectionModel()->selectedIndexes().size()) {
         onSelChanged();
     }
+
     statusText_[StatusTextNormal] = formatStatusText();
     Q_EMIT statusChanged(StatusTextNormal, statusText_[StatusTextNormal]);
 }
@@ -825,10 +902,20 @@ void TabPage::setCustomizedView(bool value) {
     }
     else { // use default folder view settings
         settings.clearFolderSettings(path());
-        setShowHidden(settings.showHidden());
-        setSortCaseSensitive(settings.sortCaseSensitive());
-        setSortFolderFirst(settings.sortFolderFirst());
-        sort(settings.sortColumn(), settings.sortOrder());
+        // settings may change temporarily by connecting to the signal TabPage::sortFilterChanged,
+        // which will be emitted below (that happens in MainWindow::onTabPageSortFilterChanged,
+        // for example), so we should remember its relevant values before proceeding
+        bool showHidden = settings.showHidden();
+        bool sortCaseSensitive = settings.sortCaseSensitive();
+        bool sortFolderFirst = settings.sortFolderFirst();
+        bool sortHiddenLast = settings.sortHiddenLast();
+        Fm::FolderModel::ColumnId sortColumn = settings.sortColumn();
+        Qt::SortOrder sortOrder = settings.sortOrder();
+        setShowHidden(showHidden);
+        setSortCaseSensitive(sortCaseSensitive);
+        setSortFolderFirst(sortFolderFirst);
+        setSortHiddenLast(sortHiddenLast);
+        sort(sortColumn, sortOrder);
     }
 }
 
