@@ -142,6 +142,11 @@ public:
     }
     void setSortFolderFirst(bool value);
 
+    bool sortHiddenLast() {
+        return folderSettings_.sortHiddenLast();
+    }
+    void setSortHiddenLast(bool value);
+
     bool sortCaseSensitive() {
         return folderSettings_.sortCaseSensitive();
     }
@@ -154,6 +159,8 @@ public:
 
     void setShowHidden(bool showHidden);
 
+    void setShowThumbnails(bool showThumbnails);
+
     void saveFolderSorting();
 
     Fm::FilePath path() {
@@ -262,7 +269,6 @@ public:
 Q_SIGNALS:
     void statusChanged(int type, QString statusText);
     void titleChanged(QString title);
-    void openDirRequested(const Fm::FilePath& path, int target);
     void sortFilterChanged();
     void forwardRequested();
     void backwardRequested();
