@@ -43,17 +43,40 @@ View::~View() {
 void View::onFileClicked(int type, const std::shared_ptr<const Fm::FileInfo>& fileInfo) {
     if(type == MiddleClick) {
         if(fileInfo->isDir()) {
-            Q_EMIT openDirRequested(fileInfo->path(), OpenInNewTab);
+            // fileInfo->path() shouldn't be used directly because
+            // it won't work in places like computer:/// or network:///
+            Fm::FileInfoList files;
+            files.emplace_back(fileInfo);
+            launchFiles(std::move(files), true);
         }
     }
     else {
-        Fm::FolderView::onFileClicked(type, fileInfo);
+        if(type == ActivatedClick) {
+            if(fileLauncher()) { // launch all selected files
+                auto files = selectedFiles();
+                if(!files.empty()) {
+                    if(files.size() > 20) {
+                        QMessageBox::StandardButton r = QMessageBox::question(window(),
+                                                        tr("Many files"),
+                                                        tr("Do you want to open these %1 files?", nullptr, files.size()).arg(files.size()),
+                                                        QMessageBox::Yes | QMessageBox::No,
+                                                        QMessageBox::No);
+                        if(r == QMessageBox::No) {
+                            return;
+                        }
+                    }
+                    launchFiles(std::move(files));
+                }
+            }
+        }
+        else {
+            Fm::FolderView::onFileClicked(type, fileInfo);
+        }
     }
 }
 
 void View::onNewWindow() {
     Fm::FileMenu* menu = static_cast<Fm::FileMenu*>(sender()->parent());
-    // FIXME: open the files in a new window
     Application* app = static_cast<Application*>(qApp);
     app->openFolders(menu->files());
 }
@@ -61,9 +84,7 @@ void View::onNewWindow() {
 void View::onNewTab() {
     Fm::FileMenu* menu = static_cast<Fm::FileMenu*>(sender()->parent());
     auto files = menu->files();
-    for(auto& file: files) {
-        Q_EMIT openDirRequested(file->path(), OpenInNewTab);
-    }
+    launchFiles(std::move(files), true);
 }
 
 void View::onOpenInTerminal() {
@@ -99,11 +120,11 @@ void View::prepareFileMenu(Fm::FileMenu* menu) {
     }
 
     if(all_directory) {
-        QAction* action = new QAction(QIcon::fromTheme("window-new"), tr("Open in New T&ab"), menu);
+        QAction* action = new QAction(QIcon::fromTheme(QStringLiteral("window-new")), tr("Open in New T&ab"), menu);
         connect(action, &QAction::triggered, this, &View::onNewTab);
         menu->insertAction(menu->separator1(), action);
 
-        action = new QAction(QIcon::fromTheme("window-new"), tr("Open in New Win&dow"), menu);
+        action = new QAction(QIcon::fromTheme(QStringLiteral("window-new")), tr("Open in New Win&dow"), menu);
         connect(action, &QAction::triggered, this, &View::onNewWindow);
         menu->insertAction(menu->separator1(), action);
 
@@ -111,7 +132,7 @@ void View::prepareFileMenu(Fm::FileMenu* menu) {
         // action = menu->addAction(_("Search"));
 
         if(all_native) {
-            action = new QAction(QIcon::fromTheme("utilities-terminal"), tr("Open in Termina&l"), menu);
+            action = new QAction(QIcon::fromTheme(QStringLiteral("utilities-terminal")), tr("Open in Termina&l"), menu);
             connect(action, &QAction::triggered, this, &View::onOpenInTerminal);
             menu->insertAction(menu->separator1(), action);
         }
@@ -140,6 +161,8 @@ void View::updateFromSettings(Settings& settings) {
 
     setAutoSelectionDelay(settings.autoSelectionDelay());
 
+    setCtrlRightClick(settings.ctrlRightClick());
+
     Fm::ProxyFolderModel* proxyModel = model();
     if(proxyModel) {
         proxyModel->setShowThumbnails(settings.showThumbnails());
@@ -147,4 +170,36 @@ void View::updateFromSettings(Settings& settings) {
     }
 }
 
+void View::launchFiles(Fm::FileInfoList files, bool inNewTabs) {
+    if(fileLauncher()) {
+        if(auto launcher = dynamic_cast<Launcher*>(fileLauncher())) {
+           // this happens on desktop
+           if(!launcher->hasMainWindow()
+              && (inNewTabs
+                  || static_cast<Application*>(qApp)->settings().singleWindowMode())) {
+                MainWindow* window = MainWindow::lastActive();
+                // if there is no last active window, find the last created window
+                if(window == nullptr) {
+                    QWidgetList windows = qApp->topLevelWidgets();
+                    for(int i = 0; i < windows.size(); ++i) {
+                        auto win = windows.at(windows.size() - 1 - i);
+                        if(win->inherits("PCManFM::MainWindow")) {
+                            window = static_cast<MainWindow*>(win);
+                            break;
+                        }
+                    }
+                }
+                auto tempLauncher = Launcher(window);
+                tempLauncher.openInNewTab();
+                tempLauncher.launchFiles(nullptr, std::move(files));
+                return;
+            }
+            if(inNewTabs) {
+                launcher->openInNewTab();
+            }
+        }
+        fileLauncher()->launchFiles(nullptr, std::move(files));
+    }
+}
+
 } // namespace PCManFM
