@@ -50,9 +50,6 @@ public:
         Fm::FolderView::setMargins(size);
     }
 
-Q_SIGNALS:
-    void openDirRequested(const Fm::FilePath& path, int target);
-
 protected Q_SLOTS:
     void onNewWindow();
     void onNewTab();
@@ -65,6 +62,7 @@ protected:
     virtual void prepareFolderMenu(Fm::FolderMenu* menu);
 
 private:
+    void launchFiles(Fm::FileInfoList files, bool inNewTabs = false);
 
 };
 
